@@ -83,7 +83,7 @@ def listify(args):
     return list(args)
 
 
-class AspObject(object):
+class AspObject:
     """Object representing a piece of ASP code."""
 
 
@@ -163,23 +163,27 @@ class Result(object):
             )
 
 
-class PyclingoDriver(object):
-    def __init__(self, cores=True, asp=None):
+class PyclingoDriver:
+    def __init__(self, cores=True, out=None):
         """Driver for the Python clingo interface.
 
         Arguments:
             cores (bool): whether to generate unsatisfiable cores for better
                 error reporting.
-            asp (file-like): optional stream to write a text-based ASP program
+            out (file-like): optional stream to write a text-based ASP program
                 for debugging or verification.
         """
         global clingo
-        self.out = asp or self.devnull()
+        if out:
+            self.out = out
+        else:
+            self.devnull()
         self.cores = cores
         self.assumptions = []
 
     def devnull(self):
-        return open(os.devnull, "w")
+        self.f = open(os.devnull, "w")
+        self.out = self.f
 
     def __exit__(self):
         self.f.close()
@@ -329,9 +333,9 @@ class PyclingoDriver(object):
         return result
 
 
-class ABICompatSolverSetup:
+class ABISolverBase:
     """
-    Class to set up and run an ABI Compatability Solver.
+    Base class with shared functions
     """
 
     def generate_elf_symbols(self, corpora, prefix=""):
@@ -343,7 +347,6 @@ class ABICompatSolverSetup:
         symbol_type("_ZN11MathLibrary10Arithmetic8MultiplyEdd", "STT_FUNC").
         symbol_binding("_ZN11MathLibrary10Arithmetic8MultiplyEdd", "STB_FUNC").
         symbol_attr("_ZN11MathLibrary10Arithmetic8MultiplyEdd", "STV_default").
-
         """
         # If we have a prefix, add a spacer
         prefix = "%s_" % prefix if prefix else ""
@@ -351,7 +354,7 @@ class ABICompatSolverSetup:
         for corpus in corpora:
             self.gen.h2("Corpus symbols: %s" % corpus.path)
 
-            for symbol, meta in corpus.elfsymbols.items():
+            for symbol, meta in corpus.symbols.items():
 
                 # It begins with a NULL symbol, not sure it's useful
                 if not symbol:
@@ -579,15 +582,21 @@ class ABICompatSolverSetup:
         data["corpus"]["needed"] = corpus.dynamic_tags.get("needed", [])
 
         # Elf symbols
-        data["corpus"]["symbols"] = corpus.elfsymbols
+        data["corpus"]["symbols"] = corpus.symbols
 
         # Add system corpora to elf symbols
         if system_libs:
             data["corpus"]["system_libs"] = {}
             for lib in self.get_system_corpora([corpus]):
-                data["corpus"]["system_libs"][lib.path] = lib.elfsymbols
+                data["corpus"]["system_libs"][lib.path] = lib.symbols
         return data
 
+
+class ABICompatSolverSetup(ABISolverBase):
+    """
+    Class to set up and run a compatibility check for a binary and two libs.
+    """
+
     def compat_setup(self, driver, corpora):
         """
         Generate an ASP program with relevant constraints for a binary
@@ -636,3 +645,43 @@ class ABICompatSolverSetup:
 
         # Add system corpora to elf symbols
         self.generate_elf_symbols(self.get_system_corpora(corpora))
+
+
+class ABICompareSolverSetup(ABISolverBase):
+    """
+    Class to set up and run a comparison between two libraries
+    """
+
+    def compat_setup(self, driver, corpora):
+        """
+        Arguments:
+            corpora: [corpusA, corpusB]
+            corpusB (corpus.Corpus): the first library corpus
+            corpusB (corpus.Corpus): the second library corpus
+        """
+        # preliminary checks
+        for corpus in corpora:
+            assert corpus.exists()
+
+        libA, libB = corpora
+
+        # driver is used by all the functions below to add facts and
+        # rules to generate an ASP program.
+        self.gen = driver
+
+        self.gen.h1("Corpus Facts")
+
+        self.gen.fact(fn.is_libA(libA.path))
+        self.gen.fact(fn.is_libB(libB.path))
+
+        # Generate high level corpus metadata facts (e.g., header)
+        self.generate_corpus_metadata(corpora)
+
+        # Dynamic libraries that are needed
+        self.generate_needed(corpora)
+
+        # generate all elf symbols (might be able to make this smaller set)
+        self.generate_elf_symbols(corpora)
+
+        # Add system corpora to elf symbols
+        self.generate_elf_symbols(self.get_system_corpora(corpora))
